"Main interface for {{ package.service_name.boto3_name }} service type defs"
from __future__ import annotations

{% with import_record_groups=package.get_type_defs_required_import_record_groups() -%}
    {% include "common/import_groups.py.jinja2" with context -%}
{% endwith -%}

{{ "\n\n" -}}

{% for typed_dict in package.typed_dicts -%}
    {% include "common/typed_dict.py.jinja2" with context -%}
    {{ "\n" -}}
{% endfor -%}
