"Main interface for {{ package.service_name.boto3_name }} service"

{% with import_record_groups=package.get_init_import_record_groups() -%}
    {% include "common/import_groups.py.jinja2" with context -%}
{% endwith -%}

{{ "\n\n" -}}

__all__ = (
{% for name in package.get_init_all_names() -%}
    {{ '"' -}}
    {{ name -}}
    {{ '"' -}}
    {{ ",\n" if not loop.last or loop.first else "\n" }}
{% endfor -%}
)
