"Main interface for {{ package.service_name.boto3_name }} service Client"
from __future__ import annotations

{% with import_record_groups=package.get_client_required_import_record_groups() -%}
    {% include "common/import_groups.py.jinja2" with context -%}
{% endwith -%}

{{ "\n\n" -}}

__all__ = (
{% for name in package.client.get_all_names() -%}
    {{ '"' -}}
    {{ name -}}
    {{ '"' -}}
    {{ ",\n" if not loop.last or loop.first else "\n" }}
{% endfor -%}
)

{{ "\n\n" -}}

{% with class=package.client -%}
    {% include "common/class.py.jinja2" with context -%}
{% endwith -%}

{{ "\n\n" -}}

{% with class=package.client.exceptions_class -%}
    {% include "common/class.py.jinja2" with context -%}
{% endwith -%}
