"Main interface for {{ package.service_name.boto3_name }} service Paginators"
from __future__ import annotations

{% with import_record_groups=package.get_paginator_required_import_record_groups() -%}
    {% include "common/import_groups.py.jinja2" with context -%}
{% endwith -%}

{{ "\n\n" -}}

__all__ = (
{% for paginator in package.paginators -%}
    {{ '"' -}}
    {{ paginator.name -}}
    {{ '"' -}}
    {{ ",\n" if not loop.last or loop.first else "\n" }}
{% endfor -%}
)

{{ "\n\n" -}}

{% for paginator in package.paginators -%}
    {% with class=paginator -%}
        {% include "common/class.py.jinja2" with context -%}
    {% endwith -%}
    {{ "\n\n\n" if not loop.last else "\n" -}}
{% endfor -%}
